using System;

namespace Smdn.Protocols.Imap4 {
  // handles '4. Data Formats'
  public class ImapData {
    public ImapDataFormat Format {
      get { return format; }
    }

    // handles '4.1. Atom'
    //         '4.2. Number'
    //         '4.3. String'
    public string Text {
      get { return text; }
    }

    // handles '4.4. Parenthesized List'
    public ImapData[] List {
      get { return list; }
    }

    private ImapData(string text)
    {
      // 4.1. Atom, 4.2. Number, 4.3. String
      this.text = text;
      this.list = null;
      this.format = ImapDataFormat.Text;
    }

    private ImapData(ImapData[] list)
    {
      // 4.4. Parenthesized List
      this.text = null;
      this.list = list;
      this.format = ImapDataFormat.List;
    }

    private ImapData()
    {
      // 4.5. NIL
      this.text = null;
      this.list = null;
      this.format = ImapDataFormat.Nil;
    }

    public static ImapData CreateTextData(string text)
    {
      if (text == null)
        throw new ArgumentNullException("text");

      return new ImapData(text);
    }

    public static ImapData CreateListData(ImapData[] list)
    {
      if (list == null)
        throw new ArgumentNullException("list");

      return new ImapData(list);
    }

    public static ImapData CreateNilData()
    {
      return new ImapData();
    }

    public override string ToString()
    {
      switch (format) {
        case ImapDataFormat.Text:
          return string.Format("{{Text={0}}}", text);
        case ImapDataFormat.List:
          return string.Format("{{List[{0}]}}", list.Length);
        case ImapDataFormat.Nil:
          return "{NIL}";
        default:
          return "{unknown}";
      }
    }

    private ImapDataFormat format = ImapDataFormat.None;
    private string text = string.Empty;
    private ImapData[] list = null;
  }
}