using System;
using System.Collections.Generic;
using System.Text;

namespace Smdn.Protocols.Imap4 {
  public class ImapMailbox {
    public ImapServerInfo ServerInfo {
      get { return serverInfo; }
    }

    public ImapUri Uri {
      get { return ImapMailboxManager.GetMailboxUrl(this); }
    }

    public string Name {
      get { return name; }
    }

    public string HierarchyDelimiter {
      get { return hierarchyDelimiter; }
      internal set { hierarchyDelimiter = value; }
    }

    public bool ReadOnly {
      get { return readOnly; }
      internal set { readOnly = value; }
    }

    public ImapMailboxFlag Flags {
      get { return flags; }
      internal set { flags = value; }
    }

    public ImapMessageFlag ApplicableFlags {
      get { return applicableFlags; }
      internal set { applicableFlags = value; }
    }

    public ImapMessageFlag PermanentFlags {
      get { return permanentFlags; }
      internal set { permanentFlags = value; }
    }

    public int ExistsMessage {
      get { return existsMessage; }
      internal set { existsMessage = value; }
    }

    public int RecentMessage {
      get { return recentMessage; }
      internal set { recentMessage = value; }
    }

    public int UnseenMessage {
      get { return unseenMessage; }
      internal set { unseenMessage = value; }
    }

    public int FirstUnseen {
      get { return firstUnseen; }
      internal set { firstUnseen = value; }
    }

    public int UidNext {
      get { return uidNext; }
      internal set { uidNext = value; }
    }

    public int UidValidity {
      get { return uidValidity; }
      internal set { uidValidity = value; }
    }

    internal ImapMailbox(ImapServerInfo serverInfo, string name)
      : this(serverInfo, name, true)
    {
    }

    internal ImapMailbox(ImapServerInfo serverInfo, string name, bool readOnly)
    {
      this.serverInfo = serverInfo;
      this.name = name;
      this.readOnly = readOnly;
    }

    public override string ToString()
    {
      return string.Format("{{Name={0}, HierarchyDelimiter={1}, ReadOnly={2}, Flags={3}, ApplicableFlags={4}, PermanentFlags={5}, ExistsMessage={6}, RecentMessage={7}, UnseenMessage={8}, FirstUnseen={9}, UidNext={10}, UidValidity={11}}}",
                           name,
                           hierarchyDelimiter,
                           readOnly,
                           flags,
                           applicableFlags,
                           permanentFlags,
                           existsMessage,
                           recentMessage,
                           unseenMessage,
                           firstUnseen,
                           uidNext,
                           uidValidity);
    }

    private readonly ImapServerInfo serverInfo;
    private readonly string name;
    private string hierarchyDelimiter = string.Empty;
    private bool readOnly;
    private ImapMailboxFlag flags = null;
    private ImapMessageFlag applicableFlags = null;
    private ImapMessageFlag permanentFlags = null;
    private int existsMessage = 0;
    private int recentMessage = 0;
    private int unseenMessage = 0;
    private int firstUnseen = 0;
    private int uidNext = 0;
    private int uidValidity = 0;
  }
}