using System;
using System.Text;

namespace Smdn.Protocols.Imap4 {
  public class ImapSequenceSet {
    // seq-number      = nz-number / "*"
    // seq-range       = seq-number ":" seq-number
    // sequence-set    = (seq-number / seq-range) *("," sequence-set)

    // 1,2,3...
    public static ImapSequenceSet Set(int number, params int[] numbers)
    {
      var sb = new StringBuilder();

      sb.Append(number.ToString());

      foreach (var n in numbers) {
        sb.Append(",");
        sb.Append(n.ToString());
      }

      return new ImapSequenceSet(sb.ToString());
    }

    public static ImapSequenceSet Set(int[] @set)
    {
      return Set(@set, 0, @set.Length);
    }

    public static ImapSequenceSet Set(int[] @set, int start, int count)
    {
      var sb = new StringBuilder();
      var end = start + count;

      sb.Append(@set[start++]);

      for (var index = start; index < end; index++) {
        sb.Append(",");
        sb.Append(@set[index].ToString());
      }

      return new ImapSequenceSet(sb.ToString());
    }

    // 1:3
    public static ImapSequenceSet Range(int @from, int to)
    {
      return new ImapSequenceSet(string.Format("{0}:{1}", @from, to));
    }

    // 1:*
    public static ImapSequenceSet From(int @from)
    {
      return new ImapSequenceSet(string.Format("{0}:*", @from));
    }

    // *:3
    public static ImapSequenceSet To(int to)
    {
      return new ImapSequenceSet(string.Format("*:{0}", to));
    }

    public ImapSequenceSet CombineWith(ImapSequenceSet @set)
    {
      return this + @set;
    }

    public static ImapSequenceSet Combine(ImapSequenceSet x, ImapSequenceSet y)
    {
      return x + y;
    }

    public static ImapSequenceSet operator+ (ImapSequenceSet x, ImapSequenceSet y)
    {
      return new ImapSequenceSet(string.Format("{0},{1}", x.sequenceSet, y.sequenceSet));
    }

    private ImapSequenceSet(string sequenceSet)
    {
      this.sequenceSet = sequenceSet;
    }

    public override string ToString()
    {
      return sequenceSet;
    }

    private readonly string sequenceSet;
  }
}