using System;

namespace Smdn.Protocols.Imap4 {
  // 7.3. Server Responses - Mailbox Size
  // 7.3.1. EXISTS Response
  // 7.3.2. RECENT Response
  // 7.4. Server Responses - Message Status
  // 7.4.1. EXPUNGE Response
  // 7.4.2. FETCH Response
  public class ImapSizeStatus {
    public ImapMessage[] Messages {
      get { return messages; }
    }

    public int[] Expunge {
      get { return expunge; }
    }

    public int? Exists {
      get { return exists; }
    }

    public int? Recent {
      get { return recent; }
    }

    internal ImapSizeStatus(ImapMessage message, int? expunge, int? exists, int? recent)
      : this((message == null) ? new ImapMessage[] {} : new[] {message},
              (expunge == null) ? new int[] {} : new[] {expunge.Value},
              exists,
              recent)
    {
    }

    public ImapSizeStatus(ImapMessage[] messages, int[] expunge, int? exists, int? recent)
    {
      if (messages == null)
        throw new ArgumentNullException("messages");
      if (expunge == null)
        throw new ArgumentNullException("expunge");

      this.messages = messages;
      this.expunge = expunge;
      this.exists = exists;
      this.recent = recent;
    }

    private readonly ImapMessage[] messages;
    private readonly int[] expunge;
    private readonly int? exists;
    private readonly int? recent;
  }
}
