using System;

namespace Smdn.Protocols.Imap4.Transaction {
  public enum ImapTransactionMethod {
    // transaction for initial server greeting
    Greeting,

    // transactions for '6.1. Client Commands - Any State'
    Capability,       // '6.1.1. CAPABILITY Command'
    NoOp,             // '6.1.2. NOOP Command'
    Logout,           // '6.1.3. LOGOUT Command'

    // transactions for '6.2. Client Commands - Not Authenticated State'
    StartTls,         // '6.2.1. STARTTLS Command'
    Authenticate,     // '6.2.2. AUTHENTICATE Command'
    Login,            // '6.2.3. LOGIN Command'

    // transactions for '6.3. Client Commands - Authenticated State'
    Select,           // '6.3.1. SELECT Command'
    Examine,          // '6.3.2. EXAMINE Command'
    Create,           // '6.3.3. CREATE Command'
    Delete,           // '6.3.4. DELETE Command'
    Rename,           // '6.3.5. RENAME Command'
    Subscribe,        // '6.3.6. SUBSCRIBE Command'
    Unsubscribe,      // '6.3.7. UNSUBSCRIBE Command'
    List,             // '6.3.8. LIST Command'
    Lsub,             // '6.3.9. LSUB Command'
    Status,           // '6.3.10. STATUS Command'
    Append,           // '6.3.11. APPEND Command'

    // transactions for '6.4. Client Commands - Selected State'
    Check,            // '6.4.1. CHECK Command'
    Close,            // '6.4.2. CLOSE Command'
    Expunge,          // '6.4.3. EXPUNGE Command'
    Search,           // '6.4.4. SEARCH Command'
    Fetch,            // '6.4.5. FETCH Command'
    Store,            // '6.4.6. STORE Command'
    Copy,             // '6.4.7. COPY Command'
    // Uid,           // '6.4.8. UID Command' -> use (with UID)Fetch, Copy, Store, Search

    // transactions for 'RFC 2177 IMAP4 IDLE command'
    // valid only in authenticated/selected state
    Idle,           
    IdleDone,         // for IDLE command continuation request responses

    // transactions for 'RFC 2342 IMAP4 Namespace'
    // valid only in authenticated/selected state
    Namespace,        // '5. NAMESPACE Command'

    // transactions for 'RFC 3691 Internet Message Access Protocol (IMAP) UNSELECT command'
    // valid only in selected state
    Unselect,         // '2. UNSELECT Command'
    
    // transactions for Gimap capability extension
    // Xlist,         // XLIST
    Xyzzy,            // XYZZY
  }
}
