using System;

namespace Smdn.Protocols.Imap4.Transaction {
  public enum ImapTransactionStatus : int {
    // 1xx Informational(not used)
    
    // 2xx Success
    Ok                    = 200,  // 成功した(OK応答)
    Done                  = 250,  // 現在のクライアントの状態では無意味な操作のためコマンドを実行しなかった
    PreAuthed             = 251,  // すでに認証済み(PREAUTH応答)
    PersonalMailboxReferral = 252, // 'RFC 2221 IMAP4 Login Referrals', a home server referral in a tagged OK response

    // 3xx Redirection
    MailboxReferral       = 350,  // 'RFC 2193 IMAP4 Mailbox Referrals' a tagged NO response
    HomeServerReferral    = 351,  // 'RFC 2221 IMAP4 Login Referrals', a home server referral in a tagged NO response, an untagged BYE

    // 4xx Client error
    BadRequest            = 400,  // 受け付けられないリクエスト
    Unauthorized          = 401,  // 現在のクライアントの状態では許可されないコマンド
    Forbidden             = 403,  // 失敗(NO応答)、または許可されない操作
    NotFound              = 404,  // URIで指定されたメールボックス、またはメッセージが見付からない
    RequestTimeout        = 408,  // リクエストがタイムアウトした、またはリクエストを中断した
    TryCreate             = 450,  // tagged NO response with TRYCREATE
    TransactionProgress   = 491,  // 他のトランザクションが進行中 <SIP 491 Request Pending>

    // 5xx Server error
    InternalServerError   = 500,  // サーバ側でのエラー(BAD応答)
    Incapable             = 501,  // サーバにそのコマンドを処理する能力がない <HTTP 501 Not Implemented>
    Disconnecting         = 550,  // サーバ側が切断しようとしている(BYE応答)
  }
}
