using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class CloseTransaction : TransactionBase {

    // ProcessClose
    //   ->WaitForClose
    // WaitForClose
    //   x Conclude

    public CloseTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      return ProcessClose;
    }

    private void ProcessClose()
    {
      Send("close", WaitForClose);
    }

    private void WaitForClose()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          Conclude(new ImapResponse(ImapTransactionStatus.Ok, "closed mailbox"));
        }
        else {
          ConcludeError(status);
        }
      }
    }
  }
}