using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class DeleteTransaction : TransactionBase {

    // ProcessDelete
    //   ->WaitForDelete
    // ProcessArgumentNotSetted
    //   x Conclude
    // WaitForDelete
    //   x Conclude

    public DeleteTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Request.Arguments.ContainsKey("mailbox name"))
        return ProcessArgumentNotSetted;
      else
        return ProcessDelete;
    }

    private void ProcessArgumentNotSetted()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "arguments 'mailbox name' must be setted"));
    }

    private void ProcessDelete()
    {
      Send("delete",
           WaitForDelete,
           new ImapQuotedString(ImapEncoding.ToModifiedUTF7String((string)Request.Arguments["mailbox name"])));
    }

    private void WaitForDelete()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          // delete mailbox instance
          ImapMailboxManager.DeleteMailbox(Connection.ServerInfo, (string)Request.Arguments["mailbox name"]);

          Conclude(new ImapResponse(ImapTransactionStatus.Ok, "mailbox deleted"));
        }
        else if (status.Status == ImapProtocolStatus.No) {
          // RFC 2193 IMAP4 Mailbox Referrals
          // http://tools.ietf.org/html/rfc2193
          // 4.1. SELECT, EXAMINE, DELETE, SUBSCRIBE, UNSUBSCRIBE, STATUS and APPEND
          //      Referrals
          ImapUri[] referrals;
          if (ImapStatusResponseConverter.FromDeleteNoResponse(status, out referrals))
            Conclude(new ImapMailboxReferralResponse(ImapTransactionStatus.MailboxReferral, referrals, status.Text));
          else
            ConcludeError(status);
        }
        else {
          ConcludeError(status);
        }
      }
    }
  }
}
