using System;
using System.Collections.Generic;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class ExpungeTransaction : TransactionBase {

    // ProcessExpunge
    //   ->WaitForExpunge
    // WaitForExpunge
    //   x Conclude

    public ExpungeTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      return ProcessExpunge;
    }

    private void ProcessExpunge()
    {
      Send("expunge", WaitForExpunge);
    }

    private void WaitForExpunge()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapDataResponse) {
        int expunge;

        if (ImapDataResponseConverter.FromExpungeResponse(response as ImapDataResponse, out expunge))
          expunged.Add(expunge);
      }
      else if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          Conclude(new ImapMessageListResponse(ImapTransactionStatus.Ok, expunged.ToArray(), string.Format("expunged {0} message(s)", expunged.Count)));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private List<int> expunged = new List<int>();
  }
}
