using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class GreetingTransaction : TransactionBase {

    // WaitForGreeting
    //   x Conclude

    public GreetingTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      Trace.Verbose("[IMAP4rev1] waiting for server greeting response");

      return WaitForGreeting;
    }

    private void WaitForGreeting()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        ImapCapability capabilities;
        ImapUri referral = null;

        if (ImapStatusResponseConverter.FromGreetingResponse(status, out capabilities, out referral)) {
          if (capabilities != null) {
            if (!capabilities.Has(ImapKnownCapabilities.Imap4Rev1)) {
              Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "server doesn't support IMAP4rev1"));
              return;
            }

            Connection.ServerInfo.Capabilities = capabilities;
          }
        }

        if (status.Status == ImapProtocolStatus.Ok) {
          Conclude(new ImapResponse(ImapTransactionStatus.Ok, "greeted"));
        }
        else if (status.Status == ImapProtocolStatus.PreAuth) {
          Conclude(new ImapResponse(ImapTransactionStatus.PreAuthed, "preauthed"));
        }
        else if (status.Status == ImapProtocolStatus.Bye && referral != null) {
          Conclude(new ImapLoginReferralResponse(ImapTransactionStatus.HomeServerReferral, referral, status.Text));
        }
        else {
          ConcludeError(status);
        }
      }
    }
  }
}
