using System;
using System.Collections.Generic;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal abstract class ListTransactionBase : TransactionBase {

    // ProcessList
    //   ->WaitForList
    // WaitForList
    //   x Conclude

    protected ListTransactionBase(string command, ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
      commandString = command.ToLower();
    }

    protected override ProcessTransactionDelegate Reset()
    {
      // if (Request.Arguments.ContainsKey("reference name"))
      //   return ProcessArgumentNotSetted;

      return ProcessList;
    }

    private void ProcessList()
    {
      ImapString listMailbox;

      if (Request.Arguments.ContainsKey("mailbox name")) {
        if (string.Empty.Equals((string)Request.Arguments["mailbox name"]))
          listMailbox = "*";
        else
          listMailbox = new ImapQuotedString(ImapEncoding.ToModifiedUTF7String((string)Request.Arguments["mailbox name"]));
      }
      else {
        listMailbox = "*";
      }

      if (Connection.ServerInfo.Capabilities.Has(ImapKnownCapabilities.Namespace)) {
        // TODO: personal以外の名前空間にあるメールボックスの考慮
        Send(commandString,
             WaitForList,
             new ImapQuotedString(Connection.ServerInfo.Namespace.Personal.Namespace),
             listMailbox);
      }
      else {
        Send(commandString,
             WaitForList,
             new ImapQuotedString(string.Empty),
             listMailbox);
      }
    }

    private void WaitForList()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapDataResponse) {
        ImapMailboxFlag flags;
        string hierarchyDelimiter;
        string name;

        if (ImapDataResponseConverter.FromListLsubResponse(response as ImapDataResponse,
                                                                   out flags,
                                                                   out hierarchyDelimiter,
                                                                   out name)) {
          // get existing or created instance
          var mailbox = ImapMailboxManager.GetMailbox(Connection.ServerInfo, name);

          mailbox.Flags = flags;
          mailbox.HierarchyDelimiter = hierarchyDelimiter;

          Trace.Dump(mailbox);

          mailboxes.Add(mailbox);
        }
        else {
          Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "mailbox comprehension error occurred"));
        }
      }
      else if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          Conclude(new ImapListResponse(ImapTransactionStatus.Ok, mailboxes.ToArray(), "mailbox listed"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string commandString = string.Empty;
    private List<ImapMailbox> mailboxes = new List<ImapMailbox>();
  }
}
