using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class LogoutTransaction : TransactionBase {

    // ProcessLogout
    //   ->WaitForLogout
    // WaitForLogout
    //   x Conclude

    public LogoutTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      return ProcessLogout;
    }

    private void ProcessLogout()
    {
      Send("logout", WaitForLogout);
    }

    private void WaitForLogout()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Status == ImapProtocolStatus.Bye) {
          // タグ無しでのBYE応答
          Conclude(new ImapResponse(ImapTransactionStatus.Disconnecting, "logged out"));
          return;
        }

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          Conclude(new ImapResponse(ImapTransactionStatus.Ok, "logged out"));
        }
        else {
          ConcludeError(status);
        }
      }
    }
  }
}
