using System;
using System.Collections.Generic;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal abstract class NoOpTransactionBase : TransactionBase {

    // ProcessNoOp
    //   ->WaitForNoOp
    // WaitForNoOp
    //   x Conclude

    protected NoOpTransactionBase(string command, ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
      commandString = command.ToLower();
    }

    protected override ProcessTransactionDelegate Reset()
    {
      return ProcessNoOp;
    }

    private void ProcessNoOp()
    {
      Send(commandString, WaitForNoOp);
    }

    private void WaitForNoOp()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapDataResponse) {
        ImapMessage messageData;
        int? expunge;
        int? exists;
        int? recent;
        if (ImapDataResponseConverter.FromNoOpCheckCommandResponse(response as ImapDataResponse,
                                                                   out messageData,
                                                                   out expunge,
                                                                   out exists,
                                                                   out recent)) {
          if (messageData != null)    updatedMessages.Add(messageData);
          if (expunge != null)        updatedExpunge.Add((int)expunge);
          if (exists != null)         updatedExists = exists;
          if (recent != null)         updatedRecent = recent;
        }
      }
      else if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          Conclude(new ImapSizeStatusUpdatedResponse(ImapTransactionStatus.Ok,
                                                new ImapSizeStatus(updatedMessages.ToArray(),
                                                                   updatedExpunge.ToArray(),
                                                                   updatedExists,
                                                                   updatedRecent),
                                                commandString + " completed"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string commandString = string.Empty;
    private readonly List<ImapMessage> updatedMessages = new List<ImapMessage>();
    private readonly List<int> updatedExpunge = new List<int>();
    private int? updatedExists;
    private int? updatedRecent;
  }
}
