using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class RenameTransaction : TransactionBase {

    // ProcessRename
    //   ->WaitForRename
    // ProcessArgumentNotSetted
    //   x Conclude
    // WaitForRename
    //   x Conclude

    public RenameTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Request.Arguments.ContainsKey("existing mailbox name"))
        return ProcessArgumentNotSetted;
      else if (!Request.Arguments.ContainsKey("new mailbox name"))
        return ProcessArgumentNotSetted;
      else
        return ProcessRename;
    }

    private void ProcessArgumentNotSetted()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "arguments 'existing mailbox name' and 'new mailbox name' must be setted"));
    }

    private void ProcessRename()
    {
      Send("rename",
           WaitForRename,
           new ImapQuotedString(ImapEncoding.ToModifiedUTF7String((string)Request.Arguments["existing mailbox name"])),
           new ImapQuotedString(ImapEncoding.ToModifiedUTF7String((string)Request.Arguments["new mailbox name"])));
    }

    private void WaitForRename()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          var renamedMailbox = ImapMailboxManager.GetRenamedMailbox(Connection.ServerInfo,
                                                                    (string)Request.Arguments["existing mailbox name"],
                                                                    (string)Request.Arguments["new mailbox name"]);

          Trace.Dump(renamedMailbox);

          Conclude(new ImapMailboxResponse(ImapTransactionStatus.Ok, renamedMailbox, "mailbox renamed"));
        }
        else if (status.Status == ImapProtocolStatus.No) {
          // RFC 2193 IMAP4 Mailbox Referrals
          // http://tools.ietf.org/html/rfc2193
          // 4.3. RENAME Referrals
          ImapUri[] referrals;
          if (ImapStatusResponseConverter.FromRenameNoResponse(status, out referrals))
            Conclude(new ImapMailboxReferralResponse(ImapTransactionStatus.MailboxReferral, referrals, status.Text));
          else
            ConcludeError(status);
        }
        else {
          ConcludeError(status);
        }
      }
    }
  }
}