using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal abstract class SubscribeTransactionBase : TransactionBase {

    // ProcessSubscribe
    //   ->WaitForSubscribe
    // ProcessArgumentNotSetted
    //   x Conclude
    // WaitForSubscribe
    //   x Conclude

    protected SubscribeTransactionBase(string command, ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
      commandString = command.ToLower();
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Request.Arguments.ContainsKey("mailbox name"))
        return ProcessArgumentNotSetted;

      return ProcessSubscribe;
    }

    private void ProcessArgumentNotSetted()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "arguments 'mailbox name' must be setted"));
    }

    private void ProcessSubscribe()
    {
      Send(commandString,
           WaitForSubscribe,
           new ImapQuotedString(ImapEncoding.ToModifiedUTF7String((string)Request.Arguments["mailbox name"])));
    }

    private void WaitForSubscribe()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          Conclude(new ImapResponse(ImapTransactionStatus.Ok, commandString + " completed"));
        }
        else if (status.Status == ImapProtocolStatus.No) {
          // RFC 2193 IMAP4 Mailbox Referrals
          // http://tools.ietf.org/html/rfc2193
          // 4.1. SELECT, EXAMINE, DELETE, SUBSCRIBE, UNSUBSCRIBE, STATUS and APPEND
          //      Referrals
          ImapUri[] referrals;
          if (ImapStatusResponseConverter.FromSubscribeUnsubscribeNoResponse(status, out referrals))
            Conclude(new ImapMailboxReferralResponse(ImapTransactionStatus.MailboxReferral, referrals, status.Text));
          else
            ConcludeError(status);
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string commandString = string.Empty;
  }
}