using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class XyzzyTransaction : TransactionBase {

    // ProcessIncalable
    //   x Conclude
    // ProcessXyzzy
    //   ->WaitForXyzzy
    //   x Conclude
    // WaitForXyzzy
    //   x Conclude

    public XyzzyTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (Connection.ServerInfo.Capabilities.Has(ImapKnownCapabilities.Xyzzy))
        return ProcessXyzzy;
      else
        return ProcessIncalable;
    }

    private void ProcessXyzzy()
    {
      Send("XYZZY", WaitForXyzzy);
    }

    private void WaitForXyzzy()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          Conclude(new ImapResponse(ImapTransactionStatus.Ok, "xyzzy!"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

  }
}
