using System;

namespace Smdn.Protocols.Pop3 {
  public class PopCapability {
    public PopKnownCapabilities KnownCapabilities {
      get { return knownCapabilities; }
      internal set { knownCapabilities = value; }
    }

    public string[] AllCapabilities {
      get { return allCapabilities; }
      internal set { allCapabilities = value; }
    }

    public string LoginDelay {
      get { return loginDelay; }
      internal set { loginDelay = value; }
    }

    public string Expire {
      get { return expire; }
      internal set { expire = value; }
    }

    internal PopCapability()
      : this(PopKnownCapabilities.None)
    {
    }

    internal PopCapability(PopKnownCapabilities knownCapabilities)
      : this(knownCapabilities, new string[] {})
    {
    }

    internal PopCapability(PopKnownCapabilities knownCapabilities, string[] allCapabilities)
    {
      this.knownCapabilities = knownCapabilities;
      this.allCapabilities = allCapabilities;
    }

    public bool Has(PopKnownCapabilities capability)
    {
      return (0 != (knownCapabilities & capability));
    }

    public bool Has(string capability)
    {
      foreach (var capa in allCapabilities) {
        if (capability.Equals(capa, StringComparison.InvariantCultureIgnoreCase))
          return true;
      }

      return false;
    }

    public override string ToString()
    {
      return string.Format("{{KnownCapabilities={0}, AllCapabilities={1}}}",
                           knownCapabilities,
                           string.Join(" ", allCapabilities));
    }

    private PopKnownCapabilities knownCapabilities;
    private string[] allCapabilities;
    private string loginDelay = null;
    private string expire = null;
  }
}
