using System;
using System.Collections.Generic;
using System.Text;

namespace Smdn.Protocols.Pop3.Protocol {
  // Layers:
  //   Client.PopClient
  //   Client.PopClientAgent
  //   Session.PopSessionAgent
  //   Session.PopSession
  //   Transaction.PopTransactionAgent
  //   Transaction.PopTransaction/Transaction.PopRequest/Transaction.PopResponse
  // * Protocol.PopConnection/Protocol.PopProtocolCommand/Protocol.PopProtocolResponse
  //   (POP3)

  // handles '6. Client Commands'
  public class PopProtocolCommand {
    public string Command {
      get { return command; }
    }

    public string[] Parameters {
      get { return parameters; }
    }

    public PopProtocolCommand(string command)
      : this(command, new string[] {})
    {
    }

    public PopProtocolCommand(string command, string[] parameters)
    {
      this.command = command;
      this.parameters = parameters;
    }

    public override string ToString()
    {
      return string.Format("{{Command={0}}}", command);
    }

    public byte[] Format()
    {
      var commandString = new StringBuilder();

      if (command != null)
        commandString.AppendFormat(command);

      for (int i = 0; i < parameters.Length; i++) {
        if (!(command == null && i == 0))
          commandString.Append(" ");
        commandString.Append(parameters[i]);
      }

      commandString.Append("\r\n");

      return Encoding.ASCII.GetBytes(commandString.ToString());
    }

    private readonly string command;
    private readonly string[] parameters;
  }
}