using System;
using System.Text;
using System.Security.Cryptography;

using Smdn.Protocols.Pop3.Protocol;

namespace Smdn.Protocols.Pop3.Transaction.Transactions {
  internal class ApopTransaction : TransactionBase {

    // ProcessApopIncapable
    //   x Conclude
    // ProcessParameterNotSetted
    //   x Conclude
    // ProcessApop
    //   ->WaitForApop
    // WaitForApop
    //   x Conclude

    public ApopTransaction(PopConnection connection, PopRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Connection.ServerInfo.Capabilities.Has(PopKnownCapabilities.Apop))
        return ProcessApopIncapable;
      else if (Request.Parameters.ContainsKey("Mailbox") && Request.Parameters.ContainsKey("Password"))
        return ProcessApop;
      else
        return ProcessParameterNotSetted;
    }

    private void ProcessApopIncapable()
    {
      Conclude(new PopResponse(PopTransactionStatus.Incapable, "APOP incapable"));
    }

    private void ProcessParameterNotSetted()
    {
      Conclude(new PopResponse(PopTransactionStatus.BadRequest, "parameter 'Password' and 'Mailbox' must be setted"));
    }

    private void ProcessApop()
    {
      Send("APOP",
           Request.Parameters["Mailbox"],
           CreateDigest(Request.Parameters["Password"], Connection.ServerInfo.Timestamp));

      Transit(WaitForApop);
    }

    private void WaitForApop()
    {
      PopProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is PopStatusResponse) {
        var status = response as PopStatusResponse;

        if (status.Status == PopProtocolStatus.Positive) {
          Conclude(new PopResponse(PopTransactionStatus.Ok, "succeeded"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string CreateDigest(string sharedSecret, string timestamp)
    {
      Console.WriteLine(timestamp + sharedSecret);
      /*
      The `digest' parameter is calculated by applying
      the MD5 algorithm [RFC1321] to a string consisting of the
      timestamp (including angle-brackets) followed by a shared
      secret.
      */
      var digest = Encoding.ASCII.GetBytes(timestamp + sharedSecret);
      var digestString = string.Empty;

      /*
      The `digest' parameter
      itself is a 16-octet value which is sent in hexadecimal
      format, using lower-case ASCII characters.
      */
      foreach (var octet in (new MD5CryptoServiceProvider()).ComputeHash(digest)) {
        digestString += octet.ToString("x02");
      }

      return digestString;
    }
  }
}
