using System;
using System.Text;

using Smdn.Protocols.Pop3.Protocol;

namespace Smdn.Protocols.Pop3.Transaction.Transactions {
  internal class RetrTransaction : TransactionBase {

    // ProcessParameterNotSetted
    //   x Conclude
    // ProcessRetr
    //   ->WaitForRetr
    // WaitForRetr
    //   x Conclude
    // RetrieveRetr
    //   x Conclude

    public RetrTransaction(PopConnection connection, PopRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Request.Parameters.ContainsKey("Message-Number"))
        return ProcessParameterNotSetted;

      messageNumber = Request.Parameters["Message-Number"];

      return ProcessRetr;
    }

    private void ProcessParameterNotSetted()
    {
      Conclude(new PopResponse(PopTransactionStatus.BadRequest, "parameter 'Message-Number' must be setted"));
    }

    private void ProcessRetr()
    {
      Send("RETR", messageNumber);

      Transit(WaitForRetr);
    }

    private void WaitForRetr()
    {
      PopProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is PopStatusResponse) {
        var status = response as PopStatusResponse;

        if (status.Status == PopProtocolStatus.Positive) {
          retrieved = new StringBuilder();
          Transit(RetrieveRetr);
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private void RetrieveRetr()
    {
      PopProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is PopFollowingResponse) {
        var following = response as PopFollowingResponse;

        if (following.Text.StartsWith("."))
          // remove byte-stuff
          retrieved.Append(following.Text.Substring(1));
        else
          retrieved.Append(following.Text);

        retrieved.Append("\r\n");
      }
      else if (response is PopTerminationResponse) {
        Conclude(new PopMessageResponse(PopTransactionStatus.Ok,
                                         new PopMessage(int.Parse(messageNumber), retrieved.ToString()),
                                         "retrieved a message"));
      }
    }

    private string messageNumber;
    private StringBuilder retrieved = null;
  }
}
